#The main module, meant to be a namespace and not a mixin.
module Randomness
	#Sets the random seed to a value based on the current time.
	def randomizeseed()
		time = Time.now()
		srand(time.yday * time.usec * time.sec)
	end	
	#Returns a random number between min and max(inclusive of both end points).
	def randint(min, max)
		return rand(max - min + 1) + min
	end
	#Returns the specified amount of random numbers between min and max(inclusive of both end points).
	#There will not be repeated numbers.
	def randints(min, max, amount)
		return randchoices((min..max).to_a(), amount)
	end
	#Returns a random selection from the list.
	def randchoice(list)
		return list[rand(list.size())]
	end
	#Returns the specified number of choices from the list. Returns nil if the amount is greater than the list size.
	#Every element in the given list will be in the returned list no more than 1 time.
	def randchoices(list, amount)
		return nil if amount > list.size()	
		
		return_list = []
		for i in 1..amount
			index = rand(list.size())
			choice = list[index]
			return_list << choice
			list.delete_at(index)
		end
		return return_list
	end
	#Returns a random number of random elements from the provided list.
	def randselection(list)
		return randchoices(list, randint(1, list.size()))
	end
	#Returns a random word from a string.
	def randword(string)
		return randchoice(string.split(' '))
	end
	#Returns the specified amount of randomly chosen words.
	def randwords(string, amount)
		return randchoices(string.split(' '), amount)
	end
	#Returns a random character from a string.
	def randchar(string)
		return randchoice(string.split(''))
	end
	#Returns the specified amount of randomly chosen characters from the string.
	def randchars(string, amount)
		return randchoices(string.split(''), amount)
	end
	#Returns a random ascii letter. 
	def randletter(uppercase=true, lowercase=true, numbers_included=false)
		letters = []
		if uppercase
			letters += ('ABCDEFGHIJKLMNOPQRSTUVWXYZ'.split(''))
		end
		if lowercase
			letters += 'abcdefghijklmnopqrstuvwxyz'.split('')
		end
		if numbers_included
			letters += '0123456789'
		end
		return randchoice(letters)
	end
	#Returns the specified amount of random ascii letters. The returned list may have repeats.
	def randletters(amount, uppercase=true, lowercase=true, numbers_included=false)
		return_list = []
		for i in 1..amount
			return_list << randletter(uppercase, lowercase, numbers_included)
		end
		return return_list
	end
	#Returns a random boolean value.
	def randbool()
		return true if randint(1, 2) == 1
		return false
	end
	#Returns the specified number of randomly chosen boolean values.
	def randbools(amount)
		return_list = []
		for i in 1..amount
			return_list << randbool()
		end
		return return_list
	end
	#Returns a random key from the given hash.
	def randhashkey(hash)
		return randchoice(hash.keys())
	end
	#Returns the specified amount of random hash keys.
	def randhashkeys(hash, amount)
		return randchoices(hash.keys(), amount)
	end
	#Returns a random value from the given hash.
	def randhashvalue(hash)
		return randchoice(hash.values())
	end
	#Returns the specified amount of random values from the given hash.
	def randhashvalues(hash, amount)
		return randchoices(hash.values(), amount)
	end
	#Shuffles the array and returns the result.
	def shufflearray(array)
		return array.shuffle()
	end
	#Shuffles the array.
	def shufflearray!(array)
		array.shuffle!()
	end
	#Shuffles the key-value mapping in the given hash and returns the result.
	def shufflehash(hash)
		return hash.values().shuffle()
	end
	#Shuffles the key-value mapping in the given hash.
	def shufflehash!(hash)
		hash.values().shuffle!()
	end
end